<?php
namespace Zeedhi\DevOps\Model\Configurationmanager\Repositories;

use Doctrine\DBAL\Connection;

class DeployStepServer extends \Doctrine\ORM\EntityRepository {
    public function findDeployStepServerInfo($deployStepId) {
        $query = <<<SQL
            SELECT DS.PRODUCT_SERVER_ID, DS.SERVER_ORDER, DP.ID AS DEPLOY_ID,
                   PS.PATH, PS.IS_ACTIVE, SR.NAME AS SERVER_NAME, SR.ID AS SERVER_ID,
                   NVL(DP.STATUS, 'P') AS STATUS, DP.DEPLOY_DATE,
                   DECODE((SELECT MIN(SERVER_ORDER)
                             FROM DEPLOY_STEP_SERVER
                            WHERE DEPLOY_STEP_ID = :DEPLOY_STEP_ID),
                          DS.SERVER_ORDER, 'Y', 'N') AS FIRST_DEPLOY_STEP,
                   DECODE((SELECT MAX(DSS.SERVER_ORDER)
                             FROM DEPLOY_STEP_SERVER DSS
                             JOIN DEPLOY DEP ON DEP.ID = DSS.DEPLOY_ID
                            WHERE DSS.DEPLOY_STEP_ID = :DEPLOY_STEP_ID
                              AND DEP.STATUS = 'D'),
                          DS.SERVER_ORDER, 'Y', 'N') AS LAST_DEPLOY_STEP,
                       (   SELECT MAX(TICKET_ID) FROM DEPLOY
                            WHERE ID = DS.DEPLOY_ID
                            AND CD_SYSTEM_TICKET = :CD_SYSTEM_TICKET
                              AND TICKET_ID IS NOT NULL
                       ) AS TICKET_ID
              FROM DEPLOY_STEP_SERVER DS
              JOIN PRODUCT_SERVER PS ON PS.ID = DS.PRODUCT_SERVER_ID
              JOIN SERVER SR ON SR.ID = PS.SERVER_ID
         LEFT JOIN DEPLOY DP ON DP.ID = DS.DEPLOY_ID
             WHERE DS.DEPLOY_STEP_ID = :DEPLOY_STEP_ID
SQL;
        $params = array (
            'DEPLOY_STEP_ID' => $deployStepId,
            'CD_SYSTEM_TICKET' => \Zeedhi\DevOps\Model\Configurationmanager\Entities\Deploy::CD_SIS_TICKET_MOVIDESK,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findCurrentDeployStepServerInfo($deployStepId, $isReapplyDeploy) {
        $statusFilter = $isReapplyDeploy ? "" : " AND NVL(DP.STATUS, 'P') <> 'D' ";
        $query = <<<SQL
                SELECT DS.PRODUCT_SERVER_ID, DS.SERVER_ORDER, DS.ID AS DEPLOY_STEP_SERVER_ID,
                       SR.ID AS SERVER_ID, SR.WSEMAIL, DP.ID AS DEPLOY_ID,
                       SR.NAME AS SERVER_NAME, SR.NAME AS SERVER,
                       PS.PATH AS BASE_PATH, NVL(DP.STATUS, 'P') AS STATUS,
                       PC.PATH_TO_DEPLOY AS PRODUCT_PATH,
                       NVL(PS.BACKUP_LIMIT, SR.BACKUP_LIMIT) AS BACKUP_LIMIT,
                       SR.IS_WINDOWS AS SERVER_IS_WINDOWS, PS.EXEC_SCRIPTS_AUTOMATIC,
                       SR.IS_REQUIRED_TICKET AS SERVER_REQUIRES_TICKET
                  FROM DEPLOY_STEP_SERVER DS
                  JOIN PRODUCT_SERVER PS ON PS.ID = DS.PRODUCT_SERVER_ID
                  JOIN SERVER SR ON SR.ID = PS.SERVER_ID
             LEFT JOIN DEPLOY DP ON DP.ID = DS.DEPLOY_ID
             LEFT JOIN PACKAGE PC ON PC.ID = PS.PACKAGE_ID
                 WHERE DS.DEPLOY_STEP_ID = :DEPLOY_STEP_ID
                       $statusFilter
              ORDER BY DS.SERVER_ORDER
SQL;
        $params = array ( 'DEPLOY_STEP_ID' => $deployStepId );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findAllDeployStepServeId($deployStepServeId) {
        $query = <<<SQL
                SELECT DS.ID AS DEPLOY_STEP_SERVER_ID ,DS.DEPLOY_STEP_ID,DS.SERVER_ORDER,DST.MAX_SCRIPT
                  FROM DEPLOY_STEP_SERVER DS
                  JOIN DEPLOY_STEP_SERVER DS2 ON DS.DEPLOY_STEP_ID = DS2.DEPLOY_STEP_ID AND DS2.ID = :DEPLOY_STEP_SERVER_ID
                  JOIN DEPLOY_STEP DST ON DST.ID = DS.DEPLOY_STEP_ID
              ORDER BY DS.SERVER_ORDER
SQL;
        $params = array ( 'DEPLOY_STEP_SERVER_ID' => $deployStepServeId );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findPreviousDeployStepServerId($deployStepId) {
        $query = <<<SQL
                SELECT ID, MAIN_DEPLOY_ID
                  FROM (SELECT DP.ID, DP.MAIN_DEPLOY_ID
                          FROM DEPLOY_STEP_SERVER DS
                          JOIN DEPLOY DP ON DP.ID = DS.DEPLOY_ID
                         WHERE DS.DEPLOY_STEP_ID = :DEPLOY_STEP_ID
                           AND DP.STATUS = 'D'
                      ORDER BY DS.SERVER_ORDER DESC)
                 WHERE ROWNUM = 1
SQL;
        $params = array ( 'DEPLOY_STEP_ID' => $deployStepId );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findDeployStepServerStatus($deployStepId, $status) {
        $query = <<<SQL
                SELECT NVL(DP.STATUS, 'P') AS STATUS,
                       DS.ID, DP.ID AS DEPLOY_ID
                  FROM DEPLOY_STEP_SERVER DS
             LEFT JOIN DEPLOY DP ON DP.ID = DS.DEPLOY_ID
                 WHERE DS.DEPLOY_STEP_ID = :DEPLOY_STEP_ID
                   AND NVL(DP.STATUS, 'P') IN (:STATUS)
SQL;
        $params = array (
            'DEPLOY_STEP_ID' => $deployStepId,
            'STATUS'         => $status
        );
        $paramType = array(
            'STATUS' => Connection::PARAM_STR_ARRAY
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params, $paramType);
    }
}
